/*
    SDL - Simple DirectMedia Layer
    Copyright (C) 1997-2009 Sam Lantinga

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

    Sam Lantinga
    slouken@libsdl.org
*/
module org.libsdl.audio;

import std.system;
import org.libsdl.stdinc;
import org.libsdl.endian;
import org.libsdl.rwops;

extern (C)
{
	struct SDL_AudioSpec
	{
		int freq;
		Uint16 format;
		Uint8  channels;
		Uint8  silence;
		Uint16 samples;
		Uint16 padding;
		Uint32 size;
		void function(void* userdata, Uint8* stream, int len) callback;
		void* userdata;
	}
}

immutable Uint16 AUDIO_U8     = 0x0008;
immutable Uint16 AUDIO_S8     = 0x8008;
immutable Uint16 AUDIO_U16LSB = 0x0010;
immutable Uint16 AUDIO_S16LSB = 0x8010;
immutable Uint16 AUDIO_U16MSB = 0x1010;
immutable Uint16 AUDIO_S16MSB = 0x9010;
immutable Uint16 AUDIO_U16    = AUDIO_U16LSB;
immutable Uint16 AUDIO_S16    = AUDIO_S16LSB;

static if (SDL_BYTEORDER == SDL_LIL_ENDIAN)
{
	immutable Uint16 AUDIO_U16SYS = AUDIO_U16LSB;
	immutable Uint16 AUDIO_S16SYS = AUDIO_S16LSB;
}
else
{
	immutable Uint16 AUDIO_U16SYS = AUDIO_U16MSB;
	immutable Uint16 AUDIO_S16SYS = AUDIO_S16MSB;
}

extern (C)
{
	struct SDL_AudioCVT
	{
		int needed;
		Uint16 src_format;
		Uint16 dst_format;
		double rate_incr;
		Uint8* buf;
		int    len;
		int    len_cvt;
		int    len_mult;
		double len_ratio;
		void function(SDL_AudioCVT* cvt, Uint16 format)[10] filters;
		int filter_index;
	}
}

extern (C)
{
	int SDL_AudioInit(const(char)* driver_name);
	void SDL_AudioQuit();

	char* SDL_AudioDriverName(char* namebuf, int maxlen);

	int SDL_OpenAudio(SDL_AudioSpec* desired, SDL_AudioSpec* obtained);
}

enum SDL_audiostatus
{
	SDL_AUDIO_STOPPED = 0,
	SDL_AUDIO_PLAYING,
	SDL_AUDIO_PAUSED
}

alias SDL_audiostatus.SDL_AUDIO_STOPPED SDL_AUDIO_STOPPED;
alias SDL_audiostatus.SDL_AUDIO_PLAYING SDL_AUDIO_PLAYING;
alias SDL_audiostatus.SDL_AUDIO_PAUSED SDL_AUDIO_PAUSED;


extern (C)
{
	SDL_audiostatus SDL_GetAudioStatus();

	void SDL_PauseAudio(int pause_on);

	SDL_AudioSpec* SDL_LoadWAV_RW(SDL_RWops* src, int freesrc, SDL_AudioSpec* spec, Uint8** audio_buf, Uint32* audio_len);
}

SDL_AudioSpec* SDL_LoadWAV(const(char)* file, SDL_AudioSpec* spec, Uint8** audio_buf, Uint32* audio_len)
{
	return SDL_LoadWAV_RW(SDL_RWFromFile(file, "rb"), 1, spec, audio_buf, audio_len);
}

extern (C)
{
	void SDL_FreeWAV(Uint8* audio_buf);

	int SDL_BuildAudioCVT(SDL_AudioCVT* cvt, Uint16 src_format, Uint8 src_channels, int src_rate, Uint16 dst_format, Uint8 dst_channels, int dst_rate);

	int SDL_ConvertAudio(SDL_AudioCVT* cvt);
}

immutable int SDL_MIX_MAXVOLUME = 128;

extern (C)
{
	void SDL_MixAudio(Uint8* dst, const(Uint8)* src, Uint32 len, int volume);

	void SDL_LockAudio();
	void SDL_UnlockAudio();

	void SDL_CloseAudio();
}
